import db_common, docopt, fiber_orm, db_postgres, sequtils, sets, strutils

import ./jira_analysispkg/jira_api

type
  Feature* = object
    id*: int
    name*: string
    epic*: int
    stories*: seq[string]
    defects*: seq[string]
    status*: string
    confidence*: int
    target_release*: string

  TmPmDb* = ref object
    conn: DbConn

  FeaturesIssue* = object
    id*: int
    featureId*: int
    issueId*: string
    issueType*: string

func connect(connString: string): TmPmDb =
  result = TmPmDb(conn: open("", "", "", connString))

generateProcsForModels(TmPmDb, [ChangeLog, Feature, Issue, FeaturesIssue, LinkedIssue])

generateLookup(TmPmDb, ChangeLog, @["historyId"])
generateLookup(TmPmDb, LinkedIssue, @["fromId", "toId", "linkType"])

when isMainModule:

  let doc = """
Usage:
  jira_analysis api-sync <url-base> <username> <api-key>
"""

  let args = docopt(doc, version = "0.2.0")
  let db = connect("host=localhost port=5500 dbname=tegra118 user=postgres password=password")

  if args["api-sync"]:
    initJiraClient($args["<url-base>"], $args["<username>"], $args["<api-key>"])
    let issuesLogsAndLinks = searchIssues(
      "project = \"UUP\" and (labels is empty or labels != \"Design&Reqs\") ORDER BY key ASC",
      includeChangelog = true
    )

    var issuesUpdated = 0
    var issuesCreated = 0
    var linksAdded = 0

    stdout.write("\nRetrieved " & $issuesLogsAndLinks[0].len & " issues. ")
    for issue in issuesLogsAndLinks[0]:
      try:
        discard db.getIssue(issue.id)
        discard db.updateIssue(issue)
        issuesUpdated += 1;
      except NotFoundError:
        discard db.createIssue(issue)
        issuesCreated += 1;
    stdout.writeLine("Created " & $issuesCreated & " and updated " & $issuesUpdated)

    stdout.write("\nFound " & $issuesLogsAndLinks[2].len & " issue links. ")
    for link in issuesLogsAndLinks[2]:
      let existingLinks = db.findLinkedIssuesByFromIdAndToIdAndLinkType(
        link.fromId, link.toId, link.linkType)
      if existingLinks.len == 0:
        discard db.createLinkedIssue(link);
        linksAdded += 1;
    stdout.writeLine("Recorded " & $linksAdded & " we didn't already have.")

    stdout.write("Retrieved " & $issuesLogsAndLinks[1].len & " change logs. ")
    let knownHistoryIds: HashSet[string] = toHashSet[string](db.getAllChangeLogs().mapIt(it.historyId))
    let newChangeLogs: seq[ChangeLog] = issuesLogsAndLinks[1].filterIt(not knownHistoryIds.contains(it.historyId))

    for changelog in newChangeLogs:
      try:
        discard db.createChangeLog(changelog)
      except NotFoundError: discard
    stdout.writeLine("Recorded " & $newChangeLogs.len & " we didn't already have.\n")
